#
# spec file for package ceph-csi
#
# Copyright (c) 2021 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


# Remove stripping of Go binaries.
%define __arch_install_post export NO_BRP_STRIP_DEBUG=true

Name:           ceph-csi
Version:        3.3.1+git0.a07260f19
Release:        1.18
Summary:        Container Storage Interface driver for Ceph block and file
License:        Apache-2.0
Group:          System/Management
URL:            https://github.com/ceph/ceph-csi

Source0:        %{name}-%{version}.tar.gz
Source1:        vendor.tar.gz
Source98:       README

%if 0%{?suse_version}
# _insert_obs_source_lines_here
ExclusiveArch:  x86_64 aarch64
%endif

# Go and spec requirements
BuildRequires:  golang(API) >= 1.13

# for go-ceph bindings
BuildRequires:  libcephfs-devel
BuildRequires:  librados-devel
BuildRequires:  librbd-devel

# Rook runtime requirements - referenced from packages installed in Rook images
# From Ceph base container: github.com/ceph/ceph-container/src/daemon-base/...
Requires:       pattern() = ceph_base

%description
Ceph CSI plugins implement an interface between CSI enabled Container
Orchestrator (CO) and Ceph cluster. It allows dynamically provisioning
Ceph block and file volumes and attaching them to workloads.

See https://github.com/ceph/ceph-csi for more information.

################################################################################
# ceph-csi helm charts
################################################################################
%package helm-charts
Summary:        Ceph CSI helm charts
Group:          System/Management
BuildArch:      noarch

%description helm-charts
Helm charts for CephFS and RBD access through ceph-csi.

################################################################################
# Build section
################################################################################

%prep
%setup -q
# make sure we use the content from the vendor tarball
rm -rf vendor/
%setup -q -T -D -a 1

# Set registry source depending on the base os type
%if 0%{?is_opensuse}
%define registry registry.opensuse.org/opensuse
%else # is SES
%if 0%{?sle_version} >= 150200
%define registry registry.suse.com/ses/7
%else
%define registry registry.suse.com/ses/6
%endif
%endif

%define cephfs_values_yaml "charts/ceph-csi-cephfs/values.yaml"
%define rbd_values_yaml "charts/ceph-csi-rbd/values.yaml"
for file in %{cephfs_values_yaml} %{rbd_values_yaml}; do
sed -i -e "s|\(.*\)quay.io.*\/\(.*\)|\1%{registry}/cephcsi/\2|" $file
sed -i -e "s|\(.*\)k8s.gcr.io.*\/\(.*\)|\1%{registry}/cephcsi/\2|" $file
done

%build

# version format is defined in _service
version_parsed=`echo %{version} | cut -d '+' -f 1`
git_commit_parsed=`echo %{version} | sed 's/.*\.//'`

go build \
  -mod=vendor \
  -buildmode=pie \
  -a \
  -ldflags " \
  -X github.com/ceph/ceph-csi/internal/util.GitCommit=$git_commit_parsed \
  -X github.com/ceph/ceph-csi/internal/util.DriverVersion=v$version_parsed" \
  -o _output/cephcsi \
  ./cmd/

%install
install --mode=755 --directory %{buildroot}%{_bindir}
install --preserve-timestamps --mode=755 --target-directory=%{buildroot}%{_bindir} _output/cephcsi

# Set versions for helm charts
helm_appVersion=`echo %{version} | cut -d '+' -f 1`
helm_version="${helm_appVersion}-%{RELEASE}"
# Set chart registry prefix for BuildTag
registry_prefix=%{registry}
registry_prefix=${registry_prefix#*/}/charts

# Install the helm charts
%define cephfs_chart_yaml "%{buildroot}%{_datadir}/%{name}-helm-charts/ceph-csi-cephfs/Chart.yaml"
%define cephfs_values_yaml "%{buildroot}%{_datadir}/%{name}-helm-charts/ceph-csi-cephfs/values.yaml"
%define rbd_chart_yaml "%{buildroot}%{_datadir}/%{name}-helm-charts/ceph-csi-rbd/Chart.yaml"
%define rbd_values_yaml "%{buildroot}%{_datadir}/%{name}-helm-charts/ceph-csi-rbd/values.yaml"
mkdir -p %{buildroot}%{_datadir}/%{name}-helm-charts/ceph-csi-{cephfs,rbd}/examples
cp -pr charts/ceph-csi-cephfs/* %{buildroot}%{_datadir}/%{name}-helm-charts/ceph-csi-cephfs
cp -pr examples/cephfs/* %{buildroot}%{_datadir}/%{name}-helm-charts/ceph-csi-cephfs/examples
cp -pr charts/ceph-csi-rbd/* %{buildroot}%{_datadir}/%{name}-helm-charts/ceph-csi-rbd
cp -pr examples/rbd/* %{buildroot}%{_datadir}/%{name}-helm-charts/ceph-csi-rbd/examples

# Set SUSE required values
for tag in latest ${helm_appVersion} ${helm_version}; do
sed -i -e "1 i\#!BuildTag: ${registry_prefix}/ceph-csi-cephfs:"${tag} %{cephfs_chart_yaml}
sed -i -e "1 i\#!BuildTag: ${registry_prefix}/ceph-csi-rbd:"${tag} %{rbd_chart_yaml}
done
sed -i -e "s|\(appVersion: \).*|\1v${helm_appVersion}|" %{cephfs_chart_yaml}
sed -i -e "s|\(appVersion: \).*|\1v${helm_appVersion}|" %{rbd_chart_yaml}
sed -i -e "s|\(version: \).*|\1${helm_version}|" %{cephfs_chart_yaml}
sed -i -e "s|\(version: \).*|\1${helm_version}|" %{rbd_chart_yaml}

# Set CSI version at build time from helm_appVersion (same as version_parsed)
sed -i -e "s|\%CSI_VERSION\%|${helm_appVersion}|" %{cephfs_values_yaml}
sed -i -e "s|\%CSI_VERSION\%|${helm_appVersion}|" %{rbd_values_yaml}

################################################################################
# Specify which files we built belong to each package
################################################################################
# ceph-csi.arch.rpm
%files
%defattr(-,root,root,-)
%{_bindir}/cephcsi

%files helm-charts
%defattr(-,root,root,-)
%doc %{_datadir}/%{name}-helm-charts/ceph-csi-cephfs/README.md
%doc %{_datadir}/%{name}-helm-charts/ceph-csi-rbd/README.md
%{_datadir}/%{name}-helm-charts

################################################################################
# Finalize
################################################################################

# ceph-csi RPMs aren't for users to install, just to be put in containers, so
# don't bother adding docs or changelog or anything

%changelog
* Fri May  7 2021 Stefan Haas <stefan.haas@suse.com>
- Update to 3.3.1
  * Build
  * Update ceph to 15.2.11
  * Fix helm chart push issue
  * RBD
  * Modified logic to check image watchers to avoid already in use issue for mirroring image
  * Return crypt error for the rpc return
* Mon Mar 29 2021 Mike Latimer <mlatimer@suse.com>
- Update to 3.2.1
  * Deployment: Fix snapshot controller deployment (#1823)
  * RBD: Fix namespace json parser (#1822)
* Thu Mar  4 2021 Stefan Haas <stefan.haas@suse.com>
- Removed Patch use_attacher_v2.1.0.patch as we use upstream
  version patterns now
- Removed Patch csi-images-SUSE.patch as we use upstream
  version patterns now
- Update to 3.2.0
  * Add new controller sidecar for omap regeneration
  * Add support for map options in RBD Storageclass
  * Add support for unmap options in RBD Storageclass
  * Update Kubernetes sidecars to latest releases
  * Update go-ceph to the latest release
  * Implement ceph fs CLI calls with go-ceph
  * Use libopenstorage/secrets API for Hashicorp Vault
  * Use luks2 for RBD encryption
  * Standardize error logs in CephFS
  * Add pod anti-affinity for provisioner Deployment
  * Make kubelet path configurable for the helm charts
  * Make CSI log level configurable for the helm charts
* Tue Dec  1 2020 Mike Latimer <mlatimer@suse.com>
- Update to 3.1.2
  * CephFS: Fix wrong error check in CreateVolume rollback action
  * CephFS: Fix issues in cephfs clone
  * CephFS: Check only the stderror message for umount
  * RBD: Add minsnapshotsonimage flag to have a soft limit for flattening
    to create clones faster on a single rbd image.
  * Journal: Fix reading omaps from objects with large key
  * CI: Fix shellcheck in test-go
* Tue Nov 24 2020 Mike Latimer <mlatimer@suse.com>
- Use csi-attacher sidecar version v2.1.0 in helm charts due to
  build challenges with v2.1.1. (No loss of functionality with this
  change.)
  + Add use_attacher_v2.1.0.patch
* Wed Nov 18 2020 Mike Latimer <mlatimer@suse.com>
- Use upstream default CSI and sidecar versions in the helm charts
  - Drop csi-images-SUSE.patch
- Add examples directory into cephfs and rbd helm charts
* Mon Oct 26 2020 Mike Latimer <mlatimer@suse.com>
- Update helm charts to use appropriate version prefix for the final
  registry destination (e.g. registry.suse.com or registry.opensuse.org)
* Mon Oct 19 2020 Mike Latimer <mlatimer@suse.com>
- Add 'latest' and appVersion tags to helm charts
* Tue Oct  6 2020 Mike Latimer <mlatimer@suse.com>
- Minor fix to helm charts to ensure SemVer formatting
* Fri Oct  2 2020 Mike Latimer <mlatimer@suse.com>
- update to 3.1.1
  * Bail out from node expansion if its block mode pvc
  * Use common logging for rbd
  * Add Upgrade E2E testing from 3.0.0 to 3.1.0
  * Update Sidecars to the latest version
  * Update go-ceph to v0.5.0
  * Implement rbd.updateVolWithImageInfo() with go-ceph
  * Add support for snapshot retention
  * Add support for "snapshot-autoprotect" feature
  * CephFS: Check error output contains Error: ENOENT
  * CephFS: Fix lock release in volumeDelete
  * CephFS: Trim spaces around fuseMountOptions
  * CephFS: Check ENOTEMPTY when removing subvolume
  * RBD: Fix cleanup in case of clone failure
  * RBD: Use stagingTargetPath in NodeExpandVolume RPC
  * RBD: Allow readonly only for volume with contentsource
  * RBD: Rados namespaced image deletion fix
* Fri Oct  2 2020 Mike Latimer <mlatimer@suse.com>
- Package ceph-csi-cephfs and ceph-csi-rbd helm charts into a new
  RPM package for later use in publishing process
  * Add BuildRequires for CSI sidecars
  * Add csi-images-SUSE.patch to set CSI sidecar versions
  * New RPM: ceph-csi-helm-charts.rpm
* Thu Sep 10 2020 Mike Latimer <mlatimer@suse.com>
- update to 3.1.0
  * Create/Delete snapshot for CephFS
  * Create PVC from CephFS snapshot
  * Create PVC from CephFS PVC
  * Add RADOS namespace support for RBD
  * E2E migration from Travis minikube to CentOS CI
  * Use common logging functions in utils
  * Add Upgrade E2E testing from 3.0.0 to 3.1.0
  * Update Sidecars to the latest version
  * Add vault creation to rbd driver deployment
  * Update E2E testing to test with latest kubernetes versions
  * Update Rook to 1.3.8 for E2E testing
  * Add backend validation of cephfs PVC in E2E
  * Make the number of CPUs for minikube VM configurable
  * Use rbdVolume.resize() with go-ceph
  * Introduce new makefile target run-e2e
  * upgrade to minikube 1.12.1
  * Debugging tool (tracevol.py) enhancement
  * Fix tracevol.py to work with the dynamic value of fsname
  * Fix tracevol.py to take config map namespace as an option
* Wed Sep  9 2020 Stefan Haas <stefan.haas@suse.com>
- Fixed spec-file (bsc#1176327)
  * ceph-csi changed internal location of variables
* Mon Aug 10 2020 Stefan Haas <stefan.haas@suse.com>
- update to 3.0.0:
  * Create/Delete snapshot for RBD
  * Create PVC from RBD snapshot
  * Create PVC from RBD PVC
  * Add support for multiple CephFS subvolume groups
  * Multi Architecture docker images(amd64 and arm64)
  * Support ROX(ReadOnlyMany) PVC for RBD
  * Support ROX(ReadOnlyMany) PVC for CephFS
  * Move to go-ceph binding from RBD CLI
  * Move to go-ceph binding from RADOS CLI
  * Add Upgrade E2E testing from 2.1.2 to 3.0.0
  * Update Sidecars to the latest version
  * Improve locking to create a parallel clone and snapshot restore
  * Simplify Error Handling
  * Update golangci-lint version in CI
  * Update gosec version in CI
  * Add support to track cephfs PVC and subvolumes
  * Introduce build.env for configuration of the environment variables
  * Update go-ceph to v0.4.0
  * Update E2E testing to test with latest kubernetes versions
  * Split out CephFS and RBD E2E tests
  * Integration with Centos CI to run containerized builds
  * Update Rook to 1.2.7 for E2E testing
  * Disable reflink when creating xfs filesystem for RBD
  * Replace klog with klog v2
  * Reduce RBAC for kubernetes sidecar containers
  * Add option to compile e2e tests in containerized
  * Add commitlint bot in CI
  * Add Stale bot to the repo
  * Add E2E and documentation for CephFS PVC
  * Update kubernetes dependency to v1.18.6
* Fri May 29 2020 Stefan Haas <stefan.haas@suse.com>
- Fixed spec-file:
  * Rook expects CSI version-string with a leading "v"
* Thu May 28 2020 Thomas Bechtold <tbechtold@suse.com>
- change version format back to make ceph-csi-image building again.
* Wed May 27 2020 Thomas Bechtold <tbechtold@suse.com>
- update to 2.1.1:
  * XFS: Superblock has unknown read-only compatible
  * XFS: Format with nouuid to avoid UUID conflict
- update to 2.1.0:
  * Add support for rbd static PVC
  * Move cephfs subvolume support from `Alpha` to `Beta`.
  * Added support for rbd topology-based provisioning.
  * Support externally managed configmap.
  * Updated Base image to ceph Octopus
  * Added csiImageKey to keep track of the image name in RADOS omap
  * Added E2E for helm charts
  * Implement CreateVolume with go-ceph which boosts performance.
  * Migrated from `dep` to `go modules`
  * Updated Kubernetes version to v1.18.0
  * Updated golang version to 1.13.9
  * Updated Kubernetes sidecar containers to latest version
  * E2E: Add Ability to test with non root userLog an error message if cephfs mounter fails during init
  * Aligned with klog standards for logging
  * Added support in to run E2E in a different namespace
  * Removed cache functionality for cephfs plugin restart
  * rbd: fallback to inline image deletion if adding it as a task fails
  * code cleanup for static errors and unwanted code blocks
  * Fix mountoption issue in  rbd
- update to 2.0.1:
  * Fix rbd mount option issue when mount options from storage-class are not
    passed to mount point
  * Added a check for cephfs subvolume present in the backend when creating
    a PVC
  * Added forcecephkernelclient as startup parameter in helm chart
  * Fix PVC delete issue when the pool is deleted
  * Fix typo in pod spec in rbd helm chart
  * Add _netdev as default mount options for both cephfs and rbd
  * Check for ENOENT errors when deleting CephFS volumes
  * Fix ceph-csi image tag version in rbd helm chart
  * Added PodSecurityPolicy support to helm charts
- drop update-tarball.sh . no longer used.
- add _service file and use common OBS source services
  * use the obscpio format for saving space on OBS
  * create a vendor.tar.gz for building
- update README for new build/update instructions
* Thu Feb  6 2020 Thomas Bechtold <tbechtold@suse.com>
- update to 2.0.0+git0.ge58d537a:
  * Added dynamic resize support for CephFS PVCs
  * Added dynamic resize support for RBD PVCs
  * Added encryption with LUKS support for RBD PVCs
  * Mutli arch support ( ceph-csi Arm64 image)
  * Upgrade documentation from v1.2.2 to v2.0.0
  * Updated code base to kube v1.17
  * leader election enabled in deployment
  * Added Version flag to cephcsi
  * Removed Kubernetes 1.13.x support with 2.0.0 release
  * CSI: run all containers as privileged in daemonset pods
  * Upgrade: csi-attacher sidecar from v1.2.0 to v2.1.0
  * Upgrade: csi-snapshotter sidecar from v1.2.1 to v1.2.2
  * Upgrade: csi-node-driver-registrar sidecar from v1.1.0 to v1.2.0
  * Upgrade: csi-resizer from sidecar v0.3.0 to v0.4.0
  * Update csi-provisioner from sidecar v1.3.0 to v1.4.0
  * Remove deprecated containerized flag in rbd
  * Discard umount error if the directory is not mounted
  * Use EmptyDir to store provisioner socket
  * Add ContentSource to the CreateVolume response
  * Rbd: only load nbd module if not available yet
  * Enhance scripts to deploy ceph cluster using rook
  * Add e2e tests for RBD resizer
  * Update minikube to latest released version
  * Update golangci-lint version to v1.21.0
  * Fix to use kubectl create not kubectl apply in the e2e
  * Add volume size roundoff for expand request
  * Add E2E for cephfs resize functionality
  * Add Documentation for PVC resize
  * Fix block resize issue in RBD
  * Add 13.0.0 Mimic supported version to the readme
  * update Metrics supported version in Readme
  * Remove hard-coded UpdateStrategy from templates
  * Add E2E for block PVC resize
  * Enable logging in E2E if the test fails
  * Enable Block E2E for rbd
  * Add ID-based logging for ExpandVolume
  * Validate rbd image name in NodeExpand
* Tue Dec  3 2019 Kristoffer Gronlund <kgronlund@suse.com>
- Update to v1.2.2:
  * Add kernel version detection for mounting with client
  * Fix issue in helm chat pushing
  * Added forcecephkernelclient as startup parameter to force enabling ceph
  * Handle EACCESS error from 'ceph rbd task add remove'
  * Fix vol size for cephfs and rbd
  * Update registration directory name
  * Reorder kernel version checking logic
  * Increase sidecar time out value to 150s.
  * Format ext4 with lazy_journal_init
  * Change default csi liveness ports to ones less common
  * Add POD_NAMESPACE to RBD provisioner deployments
  * Add a check for nil secrets
  * Remove rootfs from rbd provisioner pod
  * Added E2E for rbd nodeplugin restart
  * Deprecate containerized flag and remove nsenter
- Update to v1.2.1:
  * Change the recommended/default FS for RBD to ext4
  * Use nodiscard option while formatting RBD devices.
  * Use provisioner socket while probing liveness.
  * Reject request if the operation is in progress
  * Fix pod termination issue due to stale mount after node plugin restart.
- Remove patch (merged upstream):
  * 0001-Added-forcecephkernelclient-as-startup-parameter-to-.patch
* Thu Oct 17 2019 Richard Brown <rbrown@suse.com>
- Remove obsolete Groups tag (fate#326485)
* Wed Oct  2 2019 Kristoffer Gronlund <kgronlund@suse.com>
- Set Version when building (bsc#1152690)
* Tue Oct  1 2019 Stefan Haas <stefan.haas@suse.com>
- Add forcecephkernelclient as startup parameter (bsc#1152690)
  * Add 0001-Added-forcecephkernelclient-as-startup-parameter-to-.patch
* Mon Sep  9 2019 Blaine Gardner <blaine.gardner@suse.com>
- Update to release v1.2.0 (bsc#1151479)
  + Cephfs: Use ceph kernel client if kernel version >= 4.17
  + implement grpc metrics for ceph-csi
  + Add xfs fstype as default type in storageclass
  + Add support to use ceph manager rbd command to delete an image
  + e2e: correct log format in execCommandInPod()
  + Add 'gosec' to the static-checks
  + switch to cephfs, utils, and csicommon to new loging system
  + utility to trace backend volume from RBD pvc
  + Implement context-based logging
  + implement klog wrapper
  + unmap rbd image if connection timeout.
  + start controller or node server based on config
  + fix: Adds liveness sidecar to v1.14+ helm charts
  + Prometheus liveness probe sidecar
  + Wrap error if failed to fetch mon
  + provisioners: add reconfiguring of PID limit
  + Use "rbd device list" to list and find rbd images and their device paths
  + Update Unstage transaction to undo steps done in Stage
  + Move mounting staging instance to a sub-path within staging path
  + e2e: do not fail to delete resources when "resource not found"
  + remove post validation of rbd device
* Tue Aug 13 2019 Denis Kondratenko <denis.kondratenko@suse.com>
- Tech preview release for containers (bsc#1145433)
* Wed Jul 17 2019 Denis Kondratenko <denis.kondratenko@suse.com>
- Use ceph-base pattern instead of individual packages
* Fri Jul 12 2019 Denis Kondratenko <denis.kondratenko@suse.com>
- Updating to the v1.1.0
  + CephFS subvolume/manager based volume provisioning and deletion.
  + E2E test support for PVC creation, App pod mounting.etc.
  + CSI spec v1.1 support
  + Kube dependency updated to 1.15.
  + Configuration store change from configmap to rados omap
  + Mount options support for CephFS and RBD volumes
  + Move locks to more granular locking than CPU count based
  + Rbd support for ReadWriteMany PVCs for block mode
  + Unary plugin code for both ( CephFS and RBD ) drivers.
  + Driver name updated to CSI spec standard.
  + helm chart updates.
  + sidecar updates to latest available.
  + RBAC corrections and updates for containers.
  + Lock protection for create,delete volume ..etc operations
  + Added CSIDriver CRD.
* Fri Jul 12 2019 Denis Kondratenko <denis.kondratenko@suse.com>
- Update version to be parsed by OBS service
* Thu Jul 11 2019 Denis Kondratenko <denis.kondratenko@suse.com>
- Update version according to openSUSE package name guidelines
* Mon Jul  8 2019 Blaine Gardner <blaine.gardner@suse.com>
- Initial submission
- Build from upstream github.com/ceph/ceph-csi initially
  + Initially use master branch
- Version at commit fa68c35f3b5d9081d8aa3241baf71064af967aa9
